/*********************************************************************
 *
 *                Microchip USB C18 Firmware - Mouse Demo
 *
 *********************************************************************
 * FileName:        user_mouse.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the CompanyE for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user_mouse.h"

//*************************************YTS***********************************************
#include "timers.h"
#include "delays.h"
//***************************************************************************************

/** V A R I A B L E S ********************************************************/
#pragma udata
byte old_sw2,old_sw3;

BOOL emulate_mode;
rom signed char dir_table[]={-4,-4,-4, 0, 4, 4, 4, 0};
byte movement_length;
byte vector = 0;
char buffer[3];

//***********************
int wait=0;
//***********************
/** P R I V A T E  P R O T O T Y P E S ***************************************/
void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);
BOOL Switch3IsPressed(void);

void Emulate_Mouse(void);

//*************************************YTS***********************************************
void timer_isr (void);
//***************************************************************************************

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
//    mInitAllLEDs();
//    mInitAllSwitches();

//*************************************YTS***********************************************
TRISA = 0x04;//RA0,RA1 are outputs and RA2 is input;
PORTAbits.RA0=0;
INTCONbits.GIE =0;
//***************************************************************************************
    old_sw2 = sw2;
    old_sw3 = sw3;

    buffer[0]=0;
    buffer[1]=0;
    buffer[2]=0;
    buffer[3]=0;
    
    emulate_mode = TRUE;
}//end UserInit

/******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user routines.
 *                  It is a mixture of both USB and non-USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void ProcessIO(void)
{   
 //   BlinkUSBStatus();
//    User Application USB tasks
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;

//*************************************YTS***********************************************
if(++wait!=300) return;
wait=299;

if(INTCONbits.GIE ==0){
OpenTimer0 (TIMER_INT_ON & T0_SOURCE_INT & T0_16BIT & T0_PS_1_256);
INTCONbits.GIE = 1;
}
//***************************************************************************************

 //   if(Switch3IsPressed())
 //      emulate_mode = !emulate_mode;
 //  
    Emulate_Mouse();
    
}//end ProcessIO

void Emulate_Mouse(void)
{
    if(emulate_mode == TRUE)
    {
        if(movement_length > 14)
        {
            buffer[0] = 0;
            buffer[1] = dir_table[vector & 0x07];           // X-Vector
            buffer[2] = dir_table[(vector+2) & 0x07];       // Y-Vector
            vector++;
            movement_length = 0;
        }//end if(movement_length > 14)
    }
    else
        buffer[0] = buffer[1] = buffer[2] = 0;

    if(!mHIDTxIsBusy())
    {
        HIDTxReport(buffer,3);

//*************************************YTS***********************************************
Delay10KTCYx(240);	//200msec (in case of 48 MHz clock)
//***************************************************************************************

        movement_length++;
    }//end if(mHIDIsPutReportReady())
}//end Emulate_Mouse


//*************************************YTS***********************************************
#pragma interruptlow timer_isr save=PROD

void timer_isr (void)
{

	INTCONbits.TMR0IF = 0;
	if(PORTAbits.RA0==1){
			PORTAbits.RA0=0;
	}
	else{
			PORTAbits.RA0=1;
	}
}
//***************************************************************************************
/** EOF user_mouse.c *********************************************************/
